@@ -2,22 +2,20 @@
 # Copyright (c), Toshio Kuratomi <tkuratomi@ansible.com> 2016
 # Simplified BSD License (see licenses/simplified_bsd.txt or https://opensource.org/licenses/BSD-2-Clause)
 
-from __future__ import absolute_import, division, print_function
-__metaclass__ = type
+from __future__ import annotations
 
+import json
 import sys
 
 # Used for determining if the system is running a new enough python version
 # and should only restrict on our documented minimum versions
-_PY3_MIN = sys.version_info >= (3, 6)
-_PY2_MIN = (2, 7) <= sys.version_info < (3,)
-_PY_MIN = _PY3_MIN or _PY2_MIN
+_PY_MIN = (3, 7)
 
-if not _PY_MIN:
-    print(
-        '\n{"failed": true, '
-        '"msg": "ansible-core requires a minimum of Python2 version 2.7 or Python3 version 3.6. Current version: %s"}' % ''.join(sys.version.splitlines())
-    )
+if sys.version_info < _PY_MIN:
+    print(json.dumps(dict(
+        failed=True,
+        msg=f"ansible-core requires a minimum of Python version {'.'.join(map(str, _PY_MIN))}. Current version: {''.join(sys.version.splitlines())}",
+    )))
     sys.exit(1)
 
 # Ansible modules can be written in any language.
@@ -27,7 +25,6 @@ if not _PY_MIN:
 import __main__
 import atexit
 import errno
-import datetime
 import grp
 import fcntl
 import locale
@@ -36,17 +33,16 @@ import pwd
 import platform
 import re
 import select
+import selectors
 import shlex
 import shutil
-import signal
 import stat
 import subprocess
 import tempfile
 import time
 import traceback
-import types
 
-from itertools import chain, repeat
+from functools import reduce
 
 try:
     import syslog
@@ -74,8 +70,6 @@ except ImportError:
 # Python2 & 3 way to get NoneType
 NoneType = type(None)
 
-from ansible.module_utils.compat import selectors
-
 from ._text import to_native, to_bytes, to_text
 from ansible.module_utils.common.text.converters import (
     jsonify,
@@ -97,21 +91,9 @@ import hashlib
 
 def _get_available_hash_algorithms():
     """Return a dictionary of available hash function names and their associated function."""
-    try:
-        # Algorithms available in Python 2.7.9+ and Python 3.2+
-        # https://docs.python.org/2.7/library/hashlib.html#hashlib.algorithms_available
-        # https://docs.python.org/3.2/library/hashlib.html#hashlib.algorithms_available
-        algorithm_names = hashlib.algorithms_available
-    except AttributeError:
-        # Algorithms in Python 2.7.x (used only for Python 2.7.0 through 2.7.8)
-        # https://docs.python.org/2.7/library/hashlib.html#hashlib.hashlib.algorithms
-        algorithm_names = set(hashlib.algorithms)
-
     algorithms = {}
-
-    for algorithm_name in algorithm_names:
+    for algorithm_name in hashlib.algorithms_available:
         algorithm_func = getattr(hashlib, algorithm_name, None)
-
         if algorithm_func:
             try:
                 # Make sure the algorithm is actually available for use.
@@ -128,12 +110,6 @@ def _get_available_hash_algorithms():
 
 AVAILABLE_HASH_ALGORITHMS = _get_available_hash_algorithms()
 
-try:
-    from ansible.module_utils.common._json_compat import json
-except ImportError as e:
-    print('\n{{"msg": "Error: ansible requires the stdlib json: {0}", "failed": true}}'.format(to_native(e)))
-    sys.exit(1)
-
 from ansible.module_utils.six.moves.collections_abc import (
     KeysView,
     Mapping, MutableMapping,
@@ -144,19 +120,19 @@ from ansible.module_utils.common.locale import get_best_parsable_locale
 from ansible.module_utils.common.process import get_bin_path
 from ansible.module_utils.common.file import (
     _PERM_BITS as PERM_BITS,
-    _EXEC_PERM_BITS as EXEC_PERM_BITS,
     _DEFAULT_PERM as DEFAULT_PERM,
     is_executable,
     format_attributes,
     get_flags_from_attributes,
     FILE_ATTRIBUTES,
+    S_IXANY,
+    S_IRWU_RWG_RWO,
 )
 from ansible.module_utils.common.sys_info import (
     get_distribution,
     get_distribution_version,
     get_platform_subclass,
 )
-from ansible.module_utils.pycompat24 import get_exception, literal_eval
 from ansible.module_utils.common.parameters import (
     env_fallback,
     remove_values,
@@ -167,17 +143,6 @@ from ansible.module_utils.common.parameters import (
 )
 
 from ansible.module_utils.errors import AnsibleFallbackNotFound, AnsibleValidationErrorMultiple, UnsupportedError
-from ansible.module_utils.six import (
-    PY2,
-    PY3,
-    b,
-    binary_type,
-    integer_types,
-    iteritems,
-    string_types,
-    text_type,
-)
-from ansible.module_utils.six.moves import map, reduce, shlex_quote
 from ansible.module_utils.common.validation import (
     check_missing_parameters,
     safe_eval,
@@ -199,24 +164,6 @@ PASSWORD_MATCH = re.compile(r'^(?:.+[-_\s])?pass(?:[-_\s]?(?:word|phrase|wrd|wd)
 
 imap = map
 
-try:
-    # Python 2
-    unicode  # type: ignore[used-before-def]  # pylint: disable=used-before-assignment
-except NameError:
-    # Python 3
-    unicode = text_type
-
-try:
-    # Python 2
-    basestring  # type: ignore[used-before-def,has-type]  # pylint: disable=used-before-assignment
-except NameError:
-    # Python 3
-    basestring = string_types
-
-_literal_eval = literal_eval
-
-# End of deprecated names
-
 # Internal global holding passed in params.  This is consulted in case
 # multiple AnsibleModules are created.  Otherwise each AnsibleModule would
 # attempt to read from stdin.  Other code should not use this directly as it
@@ -373,33 +320,25 @@ def _load_params():
                 buffer = fd.read()
                 fd.close()
             else:
-                buffer = sys.argv[1]
-                if PY3:
-                    buffer = buffer.encode('utf-8', errors='surrogateescape')
+                buffer = sys.argv[1].encode('utf-8', errors='surrogateescape')
         # default case, read from stdin
         else:
-            if PY2:
-                buffer = sys.stdin.read()
-            else:
-                buffer = sys.stdin.buffer.read()
+            buffer = sys.stdin.buffer.read()
         _ANSIBLE_ARGS = buffer
 
     try:
         params = json.loads(buffer.decode('utf-8'))
     except ValueError:
-        # This helper used too early for fail_json to work.
-        print('\n{"msg": "Error: Module unable to decode valid JSON on stdin.  Unable to figure out what parameters were passed", "failed": true}')
+        # This helper is used too early for fail_json to work.
+        print('\n{"msg": "Error: Module unable to decode stdin/parameters as valid JSON. Unable to parse what parameters were passed", "failed": true}')
         sys.exit(1)
 
-    if PY2:
-        params = json_dict_unicode_to_bytes(params)
-
     try:
         return params['ANSIBLE_MODULE_ARGS']
     except KeyError:
         # This helper does not have access to fail_json so we have to print
         # json output on our own.
-        print('\n{"msg": "Error: Module unable to locate ANSIBLE_MODULE_ARGS in json data from stdin.  Unable to figure out what parameters were passed", '
+        print('\n{"msg": "Error: Module unable to locate ANSIBLE_MODULE_ARGS in JSON data from stdin. Unable to figure out what parameters were passed", '
               '"failed": true}')
         sys.exit(1)
 
@@ -492,6 +431,8 @@ class AnsibleModule(object):
 
         try:
             error = self.validation_result.errors[0]
+            if isinstance(error, UnsupportedError) and self._ignore_unknown_opts:
+                error = None
         except IndexError:
             error = None
 
@@ -568,7 +509,7 @@ class AnsibleModule(object):
             raise AssertionError("implementation error -- version and date must not both be set")
         deprecate(msg, version=version, date=date, collection_name=collection_name)
         # For compatibility, we accept that neither version nor date is set,
-        # and treat that the same as if version would haven been set
+        # and treat that the same as if version would not have been set
         if date is not None:
             self.log('[DEPRECATION WARNING] %s %s' % (msg, date))
         else:
@@ -695,7 +636,7 @@ class AnsibleModule(object):
 
     def find_mount_point(self, path):
         '''
-            Takes a path and returns it's mount point
+            Takes a path and returns its mount point
 
         :param path: a string type with a filesystem path
         :returns: the path to the mount point as a text type
@@ -891,7 +832,7 @@ class AnsibleModule(object):
                                    details=to_native(e))
 
                 if mode != stat.S_IMODE(mode):
-                    # prevent mode from having extra info orbeing invalid long number
+                    # prevent mode from having extra info or being invalid long number
                     path = to_text(b_path)
                     self.fail_json(path=path, msg="Invalid mode supplied, only permission info is allowed", details=mode)
 
@@ -968,7 +909,7 @@ class AnsibleModule(object):
             attr_mod = attributes[0]
             attributes = attributes[1:]
 
-        if existing.get('attr_flags', '') != attributes or attr_mod == '-':
+        if attributes and (existing.get('attr_flags', '') != attributes or attr_mod == '-'):
             attrcmd = self.get_bin_path('chattr')
             if attrcmd:
                 attrcmd = [attrcmd, '%s%s' % (attr_mod, attributes), b_path]
@@ -1081,7 +1022,7 @@ class AnsibleModule(object):
         if prev_mode is None:
             prev_mode = stat.S_IMODE(path_stat.st_mode)
         is_directory = stat.S_ISDIR(path_stat.st_mode)
-        has_x_permissions = (prev_mode & EXEC_PERM_BITS) > 0
+        has_x_permissions = (prev_mode & S_IXANY) > 0
         apply_X_permission = is_directory or has_x_permissions
 
         # Get the umask, if the 'user' part is empty, the effect is as if (a) were
@@ -1279,7 +1220,7 @@ class AnsibleModule(object):
                 facility = getattr(syslog, self._syslog_facility, syslog.LOG_USER)
                 syslog.openlog(str(module), 0, facility)
                 syslog.syslog(syslog.LOG_INFO, msg)
-            except TypeError as e:
+            except (TypeError, ValueError) as e:
                 self.fail_json(
                     msg='Failed to log to syslog (%s). To proceed anyway, '
                         'disable syslog logging by setting no_target_syslog '
@@ -1300,25 +1241,26 @@ class AnsibleModule(object):
                 log_args = dict()
 
             module = 'ansible-%s' % self._name
-            if isinstance(module, binary_type):
+            if isinstance(module, bytes):
                 module = module.decode('utf-8', 'replace')
 
             # 6655 - allow for accented characters
-            if not isinstance(msg, (binary_type, text_type)):
+            if not isinstance(msg, (bytes, str)):
                 raise TypeError("msg should be a string (got %s)" % type(msg))
 
             # We want journal to always take text type
             # syslog takes bytes on py2, text type on py3
-            if isinstance(msg, binary_type):
-                journal_msg = remove_values(msg.decode('utf-8', 'replace'), self.no_log_values)
+            if isinstance(msg, bytes):
+                journal_msg = msg.decode('utf-8', 'replace')
             else:
                 # TODO: surrogateescape is a danger here on Py3
-                journal_msg = remove_values(msg, self.no_log_values)
+                journal_msg = msg
 
-            if PY3:
-                syslog_msg = journal_msg
-            else:
-                syslog_msg = journal_msg.encode('utf-8', 'replace')
+            if self._target_log_info:
+                journal_msg = ' '.join([self._target_log_info, journal_msg])
+
+            # ensure we clean up secrets!
+            journal_msg = remove_values(journal_msg, self.no_log_values)
 
             if has_journal:
                 journal_args = [("MODULE", os.path.basename(__file__))]
@@ -1349,9 +1291,9 @@ class AnsibleModule(object):
                                      **dict(journal_args))
                 except IOError:
                     # fall back to syslog since logging to journal failed
-                    self._log_to_syslog(syslog_msg)
+                    self._log_to_syslog(journal_msg)
             else:
-                self._log_to_syslog(syslog_msg)
+                self._log_to_syslog(journal_msg)
 
     def _log_invocation(self):
         ''' log that ansible ran the module '''
@@ -1372,9 +1314,9 @@ class AnsibleModule(object):
                 log_args[param] = 'NOT_LOGGING_PARAMETER'
             else:
                 param_val = self.params[param]
-                if not isinstance(param_val, (text_type, binary_type)):
+                if not isinstance(param_val, (str, bytes)):
                     param_val = str(param_val)
-                elif isinstance(param_val, text_type):
+                elif isinstance(param_val, str):
                     param_val = param_val.encode('utf-8')
                 log_args[param] = heuristic_log_sanitize(param_val, self.no_log_values)
 
@@ -1520,12 +1462,7 @@ class AnsibleModule(object):
         # Add traceback if debug or high verbosity and it is missing
         # NOTE: Badly named as exception, it really always has been a traceback
         if 'exception' not in kwargs and sys.exc_info()[2] and (self._debug or self._verbosity >= 3):
-            if PY2:
-                # On Python 2 this is the last (stack frame) exception and as such may be unrelated to the failure
-                kwargs['exception'] = 'WARNING: The below traceback may *not* be related to the actual failure.\n' +\
-                                      ''.join(traceback.format_tb(sys.exc_info()[2]))
-            else:
-                kwargs['exception'] = ''.join(traceback.format_tb(sys.exc_info()[2]))
+            kwargs['exception'] = ''.join(traceback.format_tb(sys.exc_info()[2]))
 
         self.do_cleanup_files()
         self._return_formatted(kwargs)
@@ -1648,7 +1585,7 @@ class AnsibleModule(object):
         current_attribs = current_attribs.get('attr_flags', '')
         self.set_attributes_if_different(dest, current_attribs, True)
 
-    def atomic_move(self, src, dest, unsafe_writes=False):
+    def atomic_move(self, src, dest, unsafe_writes=False, keep_dest_attrs=True):
         '''atomically move src to dest, copying attributes from dest, returns true on success
         it uses os.rename to ensure this as it is an atomic operation, rest of the function is
         to work around limitations, corner cases and ensure selinux context is saved if possible'''
@@ -1656,24 +1593,11 @@ class AnsibleModule(object):
         dest_stat = None
         b_src = to_bytes(src, errors='surrogate_or_strict')
         b_dest = to_bytes(dest, errors='surrogate_or_strict')
-        if os.path.exists(b_dest):
+        if os.path.exists(b_dest) and keep_dest_attrs:
             try:
                 dest_stat = os.stat(b_dest)
-
-                # copy mode and ownership
-                os.chmod(b_src, dest_stat.st_mode & PERM_BITS)
                 os.chown(b_src, dest_stat.st_uid, dest_stat.st_gid)
-
-                # try to copy flags if possible
-                if hasattr(os, 'chflags') and hasattr(dest_stat, 'st_flags'):
-                    try:
-                        os.chflags(b_src, dest_stat.st_flags)
-                    except OSError as e:
-                        for err in 'EOPNOTSUPP', 'ENOTSUP':
-                            if hasattr(errno, err) and e.errno == getattr(errno, err):
-                                break
-                        else:
-                            raise
+                shutil.copystat(b_dest, b_src)
             except OSError as e:
                 if e.errno != errno.EPERM:
                     raise
@@ -1721,18 +1645,21 @@ class AnsibleModule(object):
                             os.close(tmp_dest_fd)
                             # leaves tmp file behind when sudo and not root
                             try:
-                                shutil.move(b_src, b_tmp_dest_name)
+                                shutil.move(b_src, b_tmp_dest_name, copy_function=shutil.copy if keep_dest_attrs else shutil.copy2)
                             except OSError:
                                 # cleanup will happen by 'rm' of tmpdir
                                 # copy2 will preserve some metadata
-                                shutil.copy2(b_src, b_tmp_dest_name)
+                                if keep_dest_attrs:
+                                    shutil.copy(b_src, b_tmp_dest_name)
+                                else:
+                                    shutil.copy2(b_src, b_tmp_dest_name)
 
                             if self.selinux_enabled():
                                 self.set_context_if_different(
                                     b_tmp_dest_name, context, False)
                             try:
                                 tmp_stat = os.stat(b_tmp_dest_name)
-                                if dest_stat and (tmp_stat.st_uid != dest_stat.st_uid or tmp_stat.st_gid != dest_stat.st_gid):
+                                if keep_dest_attrs and dest_stat and (tmp_stat.st_uid != dest_stat.st_uid or tmp_stat.st_gid != dest_stat.st_gid):
                                     os.chown(b_tmp_dest_name, dest_stat.st_uid, dest_stat.st_gid)
                             except OSError as e:
                                 if e.errno != errno.EPERM:
@@ -1758,7 +1685,7 @@ class AnsibleModule(object):
             # based on the current value of umask
             umask = os.umask(0)
             os.umask(umask)
-            os.chmod(b_dest, DEFAULT_PERM & ~umask)
+            os.chmod(b_dest, S_IRWU_RWG_RWO & ~umask)
             try:
                 os.chown(b_dest, os.geteuid(), os.getegid())
             except OSError:
@@ -1794,13 +1721,9 @@ class AnsibleModule(object):
             # create a printable version of the command for use in reporting later,
             # which strips out things like passwords from the args list
             to_clean_args = args
-            if PY2:
-                if isinstance(args, text_type):
-                    to_clean_args = to_bytes(args)
-            else:
-                if isinstance(args, binary_type):
-                    to_clean_args = to_text(args)
-            if isinstance(args, (text_type, binary_type)):
+            if isinstance(args, bytes):
+                to_clean_args = to_text(args)
+            if isinstance(args, (str, bytes)):
                 to_clean_args = shlex.split(to_clean_args)
 
             clean_args = []
@@ -1819,15 +1742,10 @@ class AnsibleModule(object):
                         is_passwd = True
                 arg = heuristic_log_sanitize(arg, self.no_log_values)
                 clean_args.append(arg)
-            self._clean = ' '.join(shlex_quote(arg) for arg in clean_args)
+            self._clean = ' '.join(shlex.quote(arg) for arg in clean_args)
 
         return self._clean
 
-    def _restore_signal_handlers(self):
-        # Reset SIGPIPE to SIG_DFL, otherwise in Python2.7 it gets ignored in subprocesses.
-        if PY2 and sys.platform != 'win32':
-            signal.signal(signal.SIGPIPE, signal.SIG_DFL)
-
     def run_command(self, args, check_rc=False, close_fds=True, executable=None, data=None, binary_data=False, path_prefix=None, cwd=None,
                     use_unsafe_shell=False, prompt_regex=None, environ_update=None, umask=None, encoding='utf-8', errors='surrogate_or_strict',
                     expand_user_and_vars=True, pass_fds=None, before_communicate_callback=None, ignore_invalid_cwd=True, handle_exceptions=True):
@@ -1904,7 +1822,7 @@ class AnsibleModule(object):
         # used by clean args later on
         self._clean = None
 
-        if not isinstance(args, (list, binary_type, text_type)):
+        if not isinstance(args, (list, bytes, str)):
             msg = "Argument 'args' to run_command must be list or string"
             self.fail_json(rc=257, cmd=args, msg=msg)
 
@@ -1913,7 +1831,7 @@ class AnsibleModule(object):
 
             # stringify args for unsafe/direct shell usage
             if isinstance(args, list):
-                args = b" ".join([to_bytes(shlex_quote(x), errors='surrogate_or_strict') for x in args])
+                args = b" ".join([to_bytes(shlex.quote(x), errors='surrogate_or_strict') for x in args])
             else:
                 args = to_bytes(args, errors='surrogate_or_strict')
 
@@ -1927,14 +1845,8 @@ class AnsibleModule(object):
                 shell = True
         else:
             # ensure args are a list
-            if isinstance(args, (binary_type, text_type)):
-                # On python2.6 and below, shlex has problems with text type
-                # On python3, shlex needs a text type.
-                if PY2:
-                    args = to_bytes(args, errors='surrogate_or_strict')
-                elif PY3:
-                    args = to_text(args, errors='surrogateescape')
-                args = shlex.split(args)
+            if isinstance(args, (bytes, str)):
+                args = shlex.split(to_text(args, errors='surrogateescape'))
 
             # expand ``~`` in paths, and all environment vars
             if expand_user_and_vars:
@@ -1944,11 +1856,8 @@ class AnsibleModule(object):
 
         prompt_re = None
         if prompt_regex:
-            if isinstance(prompt_regex, text_type):
-                if PY3:
-                    prompt_regex = to_bytes(prompt_regex, errors='surrogateescape')
-                elif PY2:
-                    prompt_regex = to_bytes(prompt_regex, errors='surrogate_or_strict')
+            if isinstance(prompt_regex, str):
+                prompt_regex = to_bytes(prompt_regex, errors='surrogateescape')
             try:
                 prompt_re = re.compile(prompt_regex, re.MULTILINE)
             except re.error:
@@ -1987,7 +1896,6 @@ class AnsibleModule(object):
             st_in = subprocess.PIPE
 
         def preexec():
-            self._restore_signal_handlers()
             if umask:
                 os.umask(umask)
 
@@ -2001,10 +1909,8 @@ class AnsibleModule(object):
             preexec_fn=preexec,
             env=env,
         )
-        if PY3 and pass_fds:
+        if pass_fds:
             kwargs["pass_fds"] = pass_fds
-        elif PY2 and pass_fds:
-            kwargs['close_fds'] = False
 
         # make sure we're in the right working directory
         if cwd:
@@ -2036,7 +1942,7 @@ class AnsibleModule(object):
             if data:
                 if not binary_data:
                     data += '\n'
-                if isinstance(data, text_type):
+                if isinstance(data, str):
                     data = to_bytes(data)
 
             selector.register(cmd.stdout, selectors.EVENT_READ)
@@ -2155,3 +2061,52 @@ class AnsibleModule(object):
 
 def get_module_path():
     return os.path.dirname(os.path.realpath(__file__))
+
+
+def __getattr__(importable_name):
+    """Inject import-time deprecation warnings."""
+    if importable_name == 'get_exception':
+        from ansible.module_utils.pycompat24 import get_exception
+        importable = get_exception
+    elif importable_name in {'literal_eval', '_literal_eval'}:
+        from ast import literal_eval
+        importable = literal_eval
+    elif importable_name == 'datetime':
+        import datetime
+        importable = datetime
+    elif importable_name == 'signal':
+        import signal
+        importable = signal
+    elif importable_name == 'types':
+        import types
+        importable = types
+    elif importable_name == 'chain':
+        from itertools import chain
+        importable = chain
+    elif importable_name == 'repeat':
+        from itertools import repeat
+        importable = repeat
+    elif importable_name in {
+        'PY2', 'PY3', 'b', 'binary_type', 'integer_types',
+        'iteritems', 'string_types', 'test_type'
+    }:
+        import importlib
+        importable = getattr(
+            importlib.import_module('ansible.module_utils.six'),
+            importable_name
+        )
+    elif importable_name == 'map':
+        importable = map
+    elif importable_name == 'shlex_quote':
+        importable = shlex.quote
+    else:
+        raise AttributeError(
+            f'cannot import name {importable_name !r} '
+            f"from '{__name__}' ({__file__ !s})"
+        )
+
+    deprecate(
+        msg=f"Importing '{importable_name}' from '{__name__}' is deprecated.",
+        version="2.21",
+    )
+    return importable
