@@ -13,8 +13,7 @@
 # You should have received a copy of the GNU General Public License
 # along with Ansible.  If not, see <http://www.gnu.org/licenses/>.
 
-from __future__ import (absolute_import, division, print_function)
-__metaclass__ = type
+from __future__ import annotations
 
 import collections
 import errno
@@ -258,7 +257,7 @@ class LinuxHardware(Hardware):
         if collected_facts.get('ansible_architecture') == 's390x':
             # getting sockets would require 5.7+ with CONFIG_SCHED_TOPOLOGY
             cpu_facts['processor_count'] = 1
-            cpu_facts['processor_cores'] = zp // zmt
+            cpu_facts['processor_cores'] = round(zp / zmt)
             cpu_facts['processor_threads_per_core'] = zmt
             cpu_facts['processor_vcpus'] = zp
             cpu_facts['processor_nproc'] = zp
@@ -283,9 +282,9 @@ class LinuxHardware(Hardware):
 
                 core_values = list(cores.values())
                 if core_values:
-                    cpu_facts['processor_threads_per_core'] = core_values[0] // cpu_facts['processor_cores']
+                    cpu_facts['processor_threads_per_core'] = round(core_values[0] / cpu_facts['processor_cores'])
                 else:
-                    cpu_facts['processor_threads_per_core'] = 1 // cpu_facts['processor_cores']
+                    cpu_facts['processor_threads_per_core'] = round(1 / cpu_facts['processor_cores'])
 
                 cpu_facts['processor_vcpus'] = (cpu_facts['processor_threads_per_core'] *
                                                 cpu_facts['processor_count'] * cpu_facts['processor_cores'])
@@ -556,6 +555,7 @@ class LinuxHardware(Hardware):
             fields = [self._replace_octal_escapes(field) for field in fields]
 
             device, mount, fstype, options = fields[0], fields[1], fields[2], fields[3]
+            dump, passno = int(fields[4]), int(fields[5])
 
             if not device.startswith(('/', '\\')) and ':/' not in device or fstype == 'none':
                 continue
@@ -563,7 +563,9 @@ class LinuxHardware(Hardware):
             mount_info = {'mount': mount,
                           'device': device,
                           'fstype': fstype,
-                          'options': options}
+                          'options': options,
+                          'dump': dump,
+                          'passno': passno}
 
             if mount in bind_mounts:
                 # only add if not already there, we might have a plain /etc/mtab
