@@ -5,8 +5,7 @@
 
 # GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)
 
-from __future__ import absolute_import, division, print_function
-__metaclass__ = type
+from __future__ import annotations
 
 
 DOCUMENTATION = '''
@@ -32,7 +31,7 @@ notes:
     - As a sanity check, downloaded key id must match the one specified.
     - "Use full fingerprint (40 characters) key ids to avoid key collisions.
       To generate a full-fingerprint imported key: C(apt-key adv --list-public-keys --with-fingerprint --with-colons)."
-    - If you specify both the key id and the URL with C(state=present), the task can verify or add the key as needed.
+    - If you specify both the key id and the URL with O(state=present), the task can verify or add the key as needed.
     - Adding a new key requires an apt cache update (e.g. using the M(ansible.builtin.apt) module's update_cache option).
 requirements:
     - gpg
@@ -44,7 +43,7 @@ options:
             - The identifier of the key.
             - Including this allows check mode to correctly report the changed state.
             - If specifying a subkey's id be aware that apt-key does not understand how to remove keys via a subkey id.  Specify the primary key's id instead.
-            - This parameter is required when C(state) is set to C(absent).
+            - This parameter is required when O(state) is set to V(absent).
         type: str
     data:
         description:
@@ -76,7 +75,7 @@ options:
         default: present
     validate_certs:
         description:
-            - If C(false), SSL certificates for the target url will not be validated. This should only be used
+            - If V(false), SSL certificates for the target url will not be validated. This should only be used
               on personally controlled sites using self-signed certificates.
         type: bool
         default: 'yes'
@@ -174,7 +173,7 @@ import os
 # FIXME: standardize into module_common
 from traceback import format_exc
 
-from ansible.module_utils._text import to_native
+from ansible.module_utils.common.text.converters import to_native
 from ansible.module_utils.basic import AnsibleModule
 from ansible.module_utils.common.locale import get_best_parsable_locale
 from ansible.module_utils.urls import fetch_url
